/*
  sdt.cpp

  Copyright (c) 2021 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#include "sdt.h"

namespace ACN {
namespace SDT {


/**
  compare to another sessionID
*/
bool SessionId::operator== (const SessionId & other) const {
  return ((other.cid == cid) &
          (other.number == number) &
          (other.protocol == protocol));
}


/**
  Constuct a sequenced channel with an owner and a direction.

  @param owner the owner of the channel
  @param direction the direction of channel communication
*/
Channel::Channel(std::shared_ptr<Component> owner, Direction direction) {
  owner_ = owner;
  direction_ = direction;
}


/**
  deconstructor that closes the channel cleanly.
*/
Channel::~Channel() {
  // TODO: close the channel.
}


/**
  Construct a new session.
*/
Session::Session() {

}


/**
  deconstructor that leaves the session gracefully.
*/
Session::~Session() {
  // TODO: Disconnect the session.
}


/**
  get the ID of the session
*/
SessionId Session::id() {
  SessionId ret;
  ret.cid = leader->cid();
  ret.number = number_;
  ret.protocol = protocol_;
  return ret;
}

Pdu::Pdu(PDU::Stream stream)
  : PDU::Pdu(stream, 1) // vectors are 1 octet
{
  // if (stream->fail()) return;
  // if (!buffer_->good()) return;
}

client_pdu_header_t::client_pdu_header_t(PDU::Stream stream) {
  protocol = stream->read32();
  association = stream->read16();
}

ClientPdu::ClientPdu(PDU::Stream stream)
  : PDU::Pdu(stream, 2) // vectors are 2 octets (MID)
{
  if (stream->fail()) return;
  if (!buffer_->good()) return;

  if (flags_.hasHeader)
    setHeader(new client_pdu_header_t(buffer_));
}

}; // SDT
}; // ACN
