/*
  AbstractStrobe.cpp

  Part of WiFlash_esp32

  Copyright (c) 2020 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#include "AbstractStrobe.h"
#include <lwip/inet.h>

/*
   Constructor
*/
AbstractStrobe::AbstractStrobe (uint16_t address)
  : AbstractFixture(address)
{
  state_ = STROBE_INACTIVE;
  profile_.intensity = 0;
  profile_.duration = 0;
  profile_.rate = 0;
}

/*
   Calculate period from DMX value
*/
uint32_t AbstractStrobe::dmxPeriod(uint8_t dmx) {
  if (dmx == 0) return 0;
  // ( millis / ( percent * scalor ) + offset ) * micros
  return (1000 / (((dmx / 255.0) * 24.5) + 0.5)) * 1000;
}

/*
   Calculate duration from DMX value
*/
uint32_t AbstractStrobe::dmxDuration(uint8_t dmx) {
  //      (( percent * scalor) + offset) * micros
  return (((dmx / 255.0) * .5 ) + .01 ) * 1000 * 1000;
}

/*
  Load data into profile from new DMX
*/
void AbstractStrobe::recvData(DMX::Universe * univ) {
  // copy data from universe to profile
  std::copy(univ->data()->begin() + address_,
            univ->data()->begin() + address_ + sizeof(profile_.dmx),
            std::begin(profile_.dmx));

  // reorder multibyte profile values (changes profile_.dmx)
  profile_.intensity = htons(profile_.intensity);
}
