/*
  AbstractStrobe.h

  Part of WiFlash_esp32

  Copyright (c) 2020 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include <stdint.h>
#include "AbstractFixture.h"
#include "../dmx/universe.h"

union strobe_profile {
  uint8_t dmx[4];
  struct {
    uint16_t intensity;
    uint8_t  duration;
    uint8_t  rate;
  } __attribute__((packed));
};

enum strobe_state {
  STROBE_INACTIVE,
  STROBE_ACTIVE_ON,
  STROBE_ACTIVE_OFF
};

class AbstractStrobe
  : public AbstractFixture
{
public:
  AbstractStrobe(uint16_t address);

  const strobe_profile * profile() {return &profile_;};
  void recvData(DMX::Universe *);     // data recieved callback

protected:
  strobe_state state_;                // state machine cycle state
  strobe_profile profile_;            // dmx data
  uint32_t dmxPeriod(uint8_t dmx);    // strobe period from dmx value (ms)
  uint32_t dmxDuration(uint8_t dmx);  // strobe duration from dmx value (ms)
};
