/*
  EspDimmer.cpp

  Part of WiFlash_esp32

  Copyright (c) 2020 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#include "EspDimmer.h"
#include "esp32-hal-ledc.h"

// ESP logging module
#include <esp_log.h>
static const char* TAG = "EspDimmer";

/*
  Constructor
*/
EspDimmer::EspDimmer(uint16_t address)
  : AbstractDimmer(address)
{
}

bool EspDimmer::begin(uint8_t pin, uint8_t pwm) {
  ESP_LOGD(TAG, "pin# %u attached to pwm# %u", pin, pwm);
  // configure hardware
  pwm_ = pwm;
  ledcAttachPin(pin, pwm_);    // Attach GPIO to PWM timer
  ledcSetup(pwm_, 2400, 15);   // 2.4KHz PWM, 15 bit resolutio
  setLevel(0);                 // LED to 0%
  return true;
}

/*
   Write a 15bit level to PWM
*/
void EspDimmer::setLevel(uint16_t level) {
  ledcWrite(pwm_, ((level >> 1) & 0xffff)); // write at 15 bit PWM
}
